#include "slider_dialog.hpp"

#include "wdg/ds_slider_painter_bevelled.hpp"
#include "wdg/ds_switch_painter_circle.hpp"
#include "wdg/ds_switch_painter_close.hpp"
#include "wdg/ds_switch_painter_svg.hpp"
#include <QDesktopWidget>
#include <QApplication>
#include <QTextStream>
#include <QPainter>
#include <QBitmap>


Slider_Dialog::Slider_Dialog(QMap<QString, QString> options) :
    QWidget(nullptr, Qt::Popup), qout(stdout)
{
 	setup_style();

	QVBoxLayout *box = new QVBoxLayout();

	_slider = new Qas_Slider( this, &_image_alloc );
	_timer = nullptr;

	int max_index = 100;

	if (options.contains("max")) {
		max_index = options["max"].toInt();
	}
	_slider->set_maximum_index(max_index);

	if (options.contains("current")) {
		_slider->set_current_index(options["current"].toInt());
	}

	if (options.contains("timer")) {
		_timer = new QTimer();
		_timer->setInterval(1000 * options["timer"].toInt());
		_timer->setSingleShot(true);
		connect(_timer, SIGNAL( timeout() ),
			qApp, SLOT( quit() ));
		_timer->start();
	}

	int slider_size ( fontMetrics().height() );
	_slider->setSizePolicy ( QSizePolicy::Fixed, QSizePolicy::Fixed );
	_slider->setFixedSize ( slider_size * 2, slider_size * 8 );
	box->addWidget ( _slider );
	box->setAlignment( _slider, Qt::AlignCenter );

	if (options.contains("icon")) {
		_lbl_icon = new QLabel;
		_lbl_icon->setMinimumSize ( pixmap_size() );
		_lbl_icon->setCursor ( Qt::PointingHandCursor );
		set_icon( QIcon::fromTheme ( options["icon"] ) );
		box->addWidget(_lbl_icon);
		box->setAlignment( _lbl_icon, Qt::AlignCenter );
		_lbl_icon->installEventFilter ( this );
	}

	setLayout( box );

	int x = 0, y = 0;
	if (options.contains("x")) {
		x = options["x"].toInt();
	}
	if (options.contains("y")) {
		y = options["y"].toInt();
	}
	setup_geometry(QPoint(x, y));

	setup_io();

	connect ( _slider, SIGNAL (sig_current_index_changed ( unsigned long )),
		  SLOT(write_position( unsigned long )) );
}


QSize
Slider_Dialog::pixmap_size ( ) const
{
	unsigned int len ( fontMetrics().height() * 5 / 2 );
	return QSize ( len, len );
}


void
Slider_Dialog::set_icon (
	const QIcon & icon_n )
{
	set_pixmap ( icon_n.pixmap ( pixmap_size() ) );
}


void
Slider_Dialog::set_pixmap (
	const QPixmap & pixmap_n )
{
	_lbl_icon->setPixmap ( pixmap_n );
}


void
Slider_Dialog::mousePressEvent (
	QMouseEvent * event_n )
{
	QWidget::mousePressEvent ( event_n );
	//if ( event_n->button() == Qt::LeftButton ) {
	QApplication::quit();
	//}
}


bool
Slider_Dialog::eventFilter (
	QObject * obj_n,
	QEvent * event_n )
{
	bool res ( QWidget::eventFilter ( obj_n, event_n ) );

	if ( event_n->type() == QEvent::MouseButtonPress ) {		
		if (obj_n == _lbl_icon ) {
			qout << "click" << endl;
			res = true;
		}
		if (_timer) {
			_timer->start();
		}
	}
	return res;
}


void
Slider_Dialog::setup_style ( )
{
	::MWdg::Mixer_Style mstyle;
	::MWdg::Mixer_Style::Style_Type stype;

	stype = ::MWdg::Mixer_Style::PLAYBACK;
	_wdg_style_db.palettes.insert (
	 	stype, mstyle.style_palette ( stype ) );

	{
		::Wdg::Painter::DS_Slider_Painter_Bevelled * pnt (
			new ::Wdg::Painter::DS_Slider_Painter_Bevelled );
		pnt->set_wdg_style_db ( &_wdg_style_db );
		_image_alloc.install_painter ( pnt );
	}

}


void
Slider_Dialog::setup_geometry ( QPoint ti_tl )
{
	QPoint npos = ti_tl;
	QSize nsize ( minimumSizeHint() );

	int x = npos.x();
	int y = npos.y();

	y = (y > 0) ? 
		y - nsize.height() :
		- y;
	x = (x > 0) ?
		x - nsize.width() :
		- x;

	npos.setX(x);
	npos.setY(y);
	move(npos);
	return;
}


void
Slider_Dialog::update_mask ( )
{
	QBitmap mask ( size() );
	mask.fill ( Qt::color0 );

	{
		QPainter pnt ( &mask );
		pnt.setRenderHints (
			QPainter::Antialiasing | QPainter::SmoothPixmapTransform );
		pnt.setPen ( Qt::NoPen );
		pnt.setBrush ( QBrush ( Qt::color1 ) );
		{
			QRectF rectf ( rect() );
			pnt.drawRoundedRect ( rectf, _corner_rad, _corner_rad );
		}
	}

	setMask ( mask );
}


void
Slider_Dialog::update_pixmap ( )
{
	QImage img ( size(), QImage::Format_ARGB32_Premultiplied );
	img.fill ( 0 );

	{
		QPainter pnt ( &img );
		pnt.setRenderHints (
			QPainter::Antialiasing | QPainter::SmoothPixmapTransform );


		{
			QPen pen ( palette().color ( QPalette::WindowText ), _border_width );
			pnt.setPen ( pen );
			QBrush brush ( palette().color ( QPalette::Window ) );
			pnt.setBrush ( brush );
		}

		{
			QRectF rectf ( rect() );
			const double adj ( _border_width / 2.0 );
			rectf.adjust ( adj, adj, -adj, -adj );
			pnt.drawRoundedRect ( rectf, _corner_rad, _corner_rad );
		}
	}

	_pxmap = QPixmap::fromImage ( img );
}


//void
//Slider_Dialog::leaveEvent (
//	QEvent * event_n )
//{
//	QWidget::leaveEvent ( event_n );
//	if ( _remains_on_hover ) {
//		_remains_on_hover = false;
//		close();
//	}
//}


void
Slider_Dialog::resizeEvent (
	QResizeEvent * event_n )
{
	_update_pxmap = true;
	QWidget::resizeEvent ( event_n );
}


void // cppcheck-suppress unusedFunction
Slider_Dialog::paintEvent (
	QPaintEvent * )
{
	if ( _update_pxmap ) {
		_update_pxmap = false;
		update_mask();
		update_pixmap();
	}

	{
		QPainter pnt ( this );
		pnt.drawPixmap ( QPoint ( 0, 0 ), _pxmap );
	}
}


void
Slider_Dialog::setup_io()
{
	notifier_in = new QSocketNotifier(0, QSocketNotifier::Read, this);
	connect( notifier_in, SIGNAL ( activated( int ) ),
		 this, SLOT ( read_position() ));
}


void
Slider_Dialog::read_position()
{
	QTextStream data_in(stdin);
	if ( ! data_in.atEnd() ) {
		update_slider(data_in.readLine().toInt());
	} else {
		notifier_in->setEnabled(false);
	}
}


void
Slider_Dialog::write_position( unsigned long v )
{
	qout << _slider->current_index() << endl;
	if (_timer) {
		_timer->start();
    }
}

void Slider_Dialog::hideEvent(QHideEvent *event)
{
    // if user click when window open, it hides
    // similar to QTBUG-20406
    QTimer::singleShot(100, this, [=]() { show(); });
}


void
Slider_Dialog::update_slider ( unsigned long v )
{
	_slider->set_current_index( v );
	qout << _slider->current_index() << endl;
}

